// Copyright 2020 Igor Tolmachev, ITSamples.com. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License"); /
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
// ==============================================================================

package com.itsamples.telnet;

import java.util.ArrayList;
import android.content.Context;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.WindowManager.LayoutParams;
import android.widget.ImageView;
import android.widget.PopupWindow;
import android.widget.TableLayout;
import android.widget.TableRow;
import android.widget.TextView;

class CustomMenu
{
    private ArrayList<CustomMenuItem> mMenuItems;
    private OnMenuItemSelectedListener mListener = null;
    private Context mContext = null;
    private LayoutInflater mLayoutInflater = null;
    private PopupWindow mPopupWindow = null;
    private boolean mIsShowing = false;
    private boolean mHideOnSelect = true;

    interface OnMenuItemSelectedListener {

        void MenuItemSelectedEvent(CustomMenuItem selection);
    }

    boolean isShowing() {

        return mIsShowing; 
    }

    void setHideOnSelect(boolean doHideOnSelect) {

        mHideOnSelect = doHideOnSelect; 
    }

    synchronized void setMenuItems(ArrayList<CustomMenuItem> items) throws Exception {

        if(mIsShowing) {
            throw new Exception("Menu list may not be modified while menu is displayed.");
        }
        mMenuItems = items;
    }

    CustomMenu(Context context, OnMenuItemSelectedListener listener, LayoutInflater lo) {

        mListener = listener;
        mMenuItems = new ArrayList<CustomMenuItem>();
        mContext = context;
        mLayoutInflater = lo;
    }

    synchronized void show(View v) {

        mIsShowing = true;
        int itemCount = mMenuItems.size();
        if (itemCount < 1)
            return; //no menu items to show
        if (mPopupWindow != null) 
            return; //already showing

        int[] nLocationArray = new int[2];
        try {
            v.getLocationOnScreen(nLocationArray);
        }
        catch (NullPointerException e) {
            e.printStackTrace();
        }

        View view = mLayoutInflater.inflate(R.layout.menu_layout, null);
        mPopupWindow = new PopupWindow(view, LayoutParams.WRAP_CONTENT,
                LayoutParams.WRAP_CONTENT, false);
        mPopupWindow.setAnimationStyle(android.R.style.Animation_Dialog);
        mPopupWindow.setOutsideTouchable(true);
        mPopupWindow.setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        mPopupWindow.showAtLocation(v, Gravity.RIGHT | Gravity.TOP, 0,
                (nLocationArray[1] + v.getHeight()));
        mPopupWindow.setTouchInterceptor(new View.OnTouchListener() {
            public boolean onTouch(View v, MotionEvent event) {
                if (event.getAction() == MotionEvent.ACTION_OUTSIDE)
                    hide();

                return false;
            }
        });

        TableLayout table = (TableLayout)view.findViewById(R.id.tableLayoutMenu);
        table.removeAllViews();

        for (int i = 0; i < itemCount; i++) {
            TableRow row = new TableRow(mContext);
            row.setLayoutParams(new LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT ));

            final CustomMenuItem cmi = mMenuItems.get(i);
            View itemLayout = mLayoutInflater.inflate(R.layout.item_menu, null);
            TextView tv = (TextView)itemLayout.findViewById(R.id.textViewCaption);
            tv.setText(cmi.getCaption());
            ImageView iv = (ImageView)itemLayout.findViewById(R.id.imageViewIcon);
            iv.setImageResource(cmi.getImageResourceId());
            itemLayout.setOnClickListener( new OnClickListener() {
               public void onClick(View v) {
                    mListener.MenuItemSelectedEvent(cmi);
                    if (mHideOnSelect)
                        hide();
               }
            });

            row.addView(itemLayout);
            table.addView(row);
        }
    }

    synchronized void hide() {

        mIsShowing = false;
        if (mPopupWindow != null) {
            mPopupWindow.dismiss();
            mPopupWindow = null;
        }
    }
}
