// Copyright 2020 Igor Tolmachev, ITSamples.com. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License"); /
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
// ==============================================================================

package com.itsamples.telnet;

import android.app.Activity;
import android.app.Dialog;
import android.content.ClipData;
import android.content.ClipboardManager;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.AsyncTask;
import android.os.Bundle;
import android.text.SpannableString;
import android.text.method.LinkMovementMethod;
import android.text.util.Linkify;
import android.util.Log;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ScrollView;
import android.widget.TextView;
import android.widget.Toast;

import org.apache.commons.net.telnet.TelnetInputListener;
import org.apache.commons.net.telnet.TelnetNotificationHandler;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.util.ArrayList;

public class MainActivity extends Activity implements TelnetInputListener, TelnetNotificationHandler,
        CustomMenu.OnMenuItemSelectedListener {

    public static final int MENU_ITEM_CLEAR     = 1;
    public static final int MENU_ITEM_HISTORY   = 2;
    public static final int MENU_ITEM_COPY      = 3;
    public static final int MENU_ITEM_EMAIL     = 4;
    public static final int MENU_ITEM_ABOUT     = 5;

    private CustomMenu mMenu;
    private Dialog mDialogAbout;
    private Dialog mDialogConnect;
    private EditText mEditTextCommand;
    private Button mButtonSend;
    private Button mButtonConnect;
    private ImageView mImageViewIcon;
    private TextView mTextViewContent;
    private TelnetConnection mConnection;
    private BufferedInputStream mInputBuffer;
    private ScrollView mScrollViewContent;

    private String mServerPort;
    private String mServerAddress;

    public static int REQUEST_HISTORY = 1;
    public static int TOAST_WARNING = 0;
    public static int TOAST_OK = 1;

    @Override
    public void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        mDialogAbout = null;
        mDialogConnect = null;
        mInputBuffer = null;

        mServerPort = "";
        mServerAddress = "";

        mConnection = new TelnetConnection();
        mConnection.getConnection().setReaderThread(true);
        mConnection.getConnection().registerInputListener(this);

        mMenu = new CustomMenu(this, this, getLayoutInflater());
        mMenu.setHideOnSelect(true);

        LoadMenuItems();

        mScrollViewContent = (ScrollView)findViewById(R.id.scrollViewContent);
        mImageViewIcon = (ImageView)findViewById(R.id.imageViewIcon);
        mTextViewContent = (TextView)findViewById(R.id.textViewContent);
        mEditTextCommand = (EditText)findViewById(R.id.editTextCommand);
        mEditTextCommand.setEnabled(false);
        mEditTextCommand.setOnKeyListener(new View.OnKeyListener() {
            public boolean onKey(View v, int keyCode, KeyEvent event) {
                // If the event is a key-down event on the "enter" button
                if (event.getAction() == KeyEvent.ACTION_DOWN &&
                        keyCode == KeyEvent.KEYCODE_ENTER) {
                    // Perform action on key press
                    sendCommand();
                    return true;
                }
                return false;
            }
        });

        mButtonSend = (Button)findViewById(R.id.buttonSend);
        mButtonSend.setEnabled(false);
        mButtonSend.setOnClickListener(new View.OnClickListener() {
            public void onClick(View arg0) {
                sendCommand();
            }
        });

        mButtonConnect = (Button)findViewById(R.id.buttonConnect);
        mButtonConnect.requestFocus();
        mButtonConnect.setOnClickListener(new View.OnClickListener() {
            public void onClick(View arg0) {
                if(!mConnection.isConnected()) {
                    showConnectDialog();
                }
                else {
                    mConnection.disconnect();
                    updateConnectionStatus(false);
                }
            }
        });

        ImageView imageViewMenu = (ImageView)findViewById(R.id.imageViewMenu);
        imageViewMenu.setOnClickListener(new View.OnClickListener() {
            public void onClick(View arg0) {
                if (mMenu.isShowing())
                    mMenu.hide();
                else
                    mMenu.show(findViewById(R.id.layoutActionBar));
            }
        });

//        int SDK_INT = android.os.Build.VERSION.SDK_INT;
//        if (SDK_INT > 8)
//        {
//            StrictMode.ThreadPolicy policy = new StrictMode.ThreadPolicy.Builder()
//                    .permitAll().build();
//            StrictMode.setThreadPolicy(policy);
//        }
    }

    @Override
    public void onConfigurationChanged(Configuration newConfig) {

        if (mMenu.isShowing())
            mMenu.hide();

        super.onConfigurationChanged(newConfig);
    }

    @Override
    public void onPause() {

        super.onPause();

        if(mDialogAbout != null) {
            mDialogAbout.dismiss();
            mDialogAbout = null;
        }

        if(mDialogConnect != null) {
            mDialogConnect.dismiss();
            mDialogConnect = null;
        }
    }

    public void telnetInputAvailable() {

        if(mConnection.isConnected()) {
            try {
                while (mInputBuffer.available() > 0) {
                    final String s = Character.toString((char) mInputBuffer.read());
                    runOnUiThread(new Runnable() {
                        public void run() {
                            mTextViewContent.append(s);
                        }
                    });
                }

                mScrollViewContent.post(new Runnable() {
                    public void run() {
                        mScrollViewContent.fullScroll(View.FOCUS_DOWN);
                    }
                });
            }
            catch (IOException e) {
                Log.d("TINYY", e.getMessage());
                e.printStackTrace();
            }
        }
    }

    public void receivedNegotiation(int negotiation_code, int option_code) {

        final String strCommand;
        switch(negotiation_code) {
            case RECEIVED_DO:
                strCommand = "RECEIVED_DO\n";
                break;
            case RECEIVED_DONT:
                strCommand = "RECEIVED_DONT\n";
                break;
            case RECEIVED_WILL:
                strCommand = "RECEIVED_WILL\n";
                break;
            case RECEIVED_WONT:
                strCommand = "RECEIVED_WONT\n";
                break;
            case RECEIVED_COMMAND:
                strCommand = "RECEIVED_COMMAND\n";
                break;
            default:
                strCommand = "\n";
                break;
        }

        runOnUiThread(new Runnable() {
            public void run() {
                mTextViewContent.append(strCommand);
            }
        });
    }

    public boolean onKeyDown(int keyCode, KeyEvent event) {

        if (keyCode == KeyEvent.KEYCODE_MENU) {
            if (mMenu.isShowing())
                mMenu.hide();
            else
                mMenu.show(findViewById(R.id.layoutActionBar));

            return true;
        }

        return super.onKeyDown(keyCode, event);
    }

    private void LoadMenuItems() {

        ArrayList<CustomMenuItem> menuItems = new ArrayList<CustomMenuItem>();
        CustomMenuItem cmi = new CustomMenuItem();
        cmi.setCaption(getResources().getString(R.string.stringClear));
        cmi.setImageResourceId(R.drawable.clear);
        cmi.setId(MENU_ITEM_CLEAR);
        menuItems.add(cmi);
        cmi = new CustomMenuItem();
        cmi.setCaption(getResources().getString(R.string.stringHistory));
        cmi.setImageResourceId(R.drawable.history);
        cmi.setId(MENU_ITEM_HISTORY);
        menuItems.add(cmi);
        cmi = new CustomMenuItem();
        cmi.setCaption(getResources().getString(R.string.stringCopy));
        cmi.setImageResourceId(R.drawable.copy);
        cmi.setId(MENU_ITEM_COPY);
        menuItems.add(cmi);
        cmi = new CustomMenuItem();
        cmi.setCaption(getResources().getString(R.string.stringEmail));
        cmi.setImageResourceId(R.drawable.mail);
        cmi.setId(MENU_ITEM_EMAIL);
        menuItems.add(cmi);
        cmi = new CustomMenuItem();
        cmi.setCaption(getResources().getString(R.string.stringAbout));
        cmi.setImageResourceId(R.drawable.info);
        cmi.setId(MENU_ITEM_ABOUT);
        menuItems.add(cmi);

        if (!mMenu.isShowing()) {
            try {
                mMenu.setMenuItems(menuItems);
            }
            catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public void MenuItemSelectedEvent(CustomMenuItem item) {

        final String strText = mTextViewContent.getText().toString();
        switch (item.getId()) {
            case MENU_ITEM_CLEAR:
                mTextViewContent.setText("");
                break;

            case MENU_ITEM_HISTORY:
                Intent iHistory = new Intent(MainActivity.this, HistoryActivity.class);
                startActivityForResult(iHistory, REQUEST_HISTORY);
                break;

            case MENU_ITEM_COPY:
                if(strText.equals("")) {
                    showToast(getResources().getString(R.string.stringNothingCopy), TOAST_WARNING);
                    break;
                }

                if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.HONEYCOMB) {
                    ClipboardManager clipboard = (ClipboardManager) getSystemService(Context.CLIPBOARD_SERVICE);
                    ClipData clip = ClipData.newPlainText(strText, strText);
                    clipboard.setPrimaryClip(clip);
                }
                else  {
                    android.text.ClipboardManager clipboard =
                            (android.text.ClipboardManager) getSystemService(Context.CLIPBOARD_SERVICE);
                    clipboard.setText(strText);
                }

                showToast(getResources().getString(R.string.stringCopied), TOAST_OK);
                break;
            case MENU_ITEM_EMAIL:
                if(strText.equals("")) {
                    showToast(getResources().getString(R.string.stringNothingSend), TOAST_WARNING);
                    break;
                }

                final Intent emailIntent = new Intent(android.content.Intent.ACTION_SEND);
                emailIntent.setType("plain/text");
                //emailIntent.putExtra(android.content.Intent.EXTRA_EMAIL, new String[]{ address.getText().toString()});
                emailIntent.putExtra(android.content.Intent.EXTRA_SUBJECT, "TinyTelnet");
                emailIntent.putExtra(android.content.Intent.EXTRA_TEXT, strText);
                startActivity(Intent.createChooser(emailIntent, getResources().getString(R.string.stringSendEmail)));
                break;

            case MENU_ITEM_ABOUT:
                final SpannableString spinnableString = new SpannableString(getResources().getString(R.string.stringHomeLink));
                Linkify.addLinks(spinnableString, Linkify.ALL);

                mDialogAbout = new Dialog(this);
                mDialogAbout.requestWindowFeature(Window.FEATURE_NO_TITLE);
                mDialogAbout.setCancelable(true);
                mDialogAbout.setContentView(R.layout.dialog_about);
                Window window = mDialogAbout.getWindow();
                if(window != null)
                    window.setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));

                Button buttonOK = (Button)mDialogAbout.findViewById(R.id.buttonOK);
                buttonOK.setOnClickListener(new View.OnClickListener() {
                    public void onClick(View v) {
                        mDialogAbout.dismiss();
                    }
                });

                mDialogAbout.show();

                // Make the textview clickable. Must be called after show()
                TextView textViewHomeLink = (TextView)mDialogAbout.findViewById(R.id.textViewHomeLink);
                if(textViewHomeLink != null) {
                    textViewHomeLink.setLinkTextColor(Color.parseColor("#009688"));
                    textViewHomeLink.setText(spinnableString);
                    textViewHomeLink.setMovementMethod(LinkMovementMethod.getInstance());
                }

                break;
        }
    }

    private void sendCommand() {

        final String strCommand = mEditTextCommand.getText().toString();
        if (strCommand.length() < 1) {
            messageBox(getResources().getString(R.string.stringPleaseExecute));
        }
        else {
            new Thread("SEND"){
                public void run(){
                    if(mConnection.sendCommand(strCommand + "\n")) {
                        mEditTextCommand.setText("");
                    }
                }
            }.start();

            SharedPreferences settings = getSharedPreferences("TinyTelnet", 0);
            SharedPreferences.Editor editor = settings.edit();

            // load history list
            String strAllHistory = settings.getString("history", "");
            if(!strAllHistory.equals("")) {
                if(strAllHistory.contains(",")) {
                    String strNewHistory = strCommand;

                    String[] strHistoryItems = strAllHistory.split(",");
                    for (String strItem : strHistoryItems) {
                        if(!strItem.equals("") && !strItem.equals(strCommand)) {
                            if(!strNewHistory.equals(""))
                                strNewHistory += ",";
                            strNewHistory += strItem;
                        }
                    }

                    if(!strNewHistory.equals(""))
                        editor.putString("history", strNewHistory);
                }
                else {
                    // if saved call in history is different
                    if(!strAllHistory.equals(strCommand))
                        editor.putString("history", strCommand + "," + strAllHistory);
                }
            }
            else
                editor.putString("history", strCommand);

            editor.apply();
        }
    }

    public synchronized void onActivityResult(final int requestCode, int resultCode, final Intent data) {

        if (resultCode == Activity.RESULT_OK && requestCode == REQUEST_HISTORY) {
            String strItem = data.getStringExtra(HistoryActivity.RESULT_COMMAND);
            if(!strItem.equals("")) {
                mEditTextCommand.setText(strItem);
                sendCommand();
            }
        }
    }

    private void messageBox(String strText) {

        final Dialog dialog = new Dialog(this);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCancelable(true);
        dialog.setContentView(R.layout.dialog_ok);
        Window window = dialog.getWindow();
        if(window != null)
            window.setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));

        TextView textViewMessage = (TextView)dialog.findViewById(R.id.textViewMessage);
        textViewMessage.setText(strText);

        Button buttonOK = (Button)dialog.findViewById(R.id.buttonOK);
        buttonOK.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                dialog.dismiss();
            }
        });

        dialog.show();
    }

    private void showToast(String strMessage, int nButton) {

        LayoutInflater inflater = getLayoutInflater();
        View view = inflater.inflate(R.layout.toast, (ViewGroup) findViewById(R.id.relativeLayoutToast));

        TextView textViewMessage = (TextView)view.findViewById(R.id.textViewMessage);
        if(textViewMessage != null)
            textViewMessage.setText(strMessage);

        ImageView imageViewIcon = (ImageView)view.findViewById(R.id.imageViewIcon);
        if(imageViewIcon != null && nButton == TOAST_OK)
            imageViewIcon.setImageResource(R.drawable.ok);

        Toast toast = new Toast(this);
        toast.setView(view);
        toast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);
        toast.setDuration(Toast.LENGTH_LONG);
        toast.show();
    }

    private void updateConnectionStatus(boolean connected) {

        if(connected) {
            mButtonConnect.setText(R.string.stringDisconnect);
            mImageViewIcon.setImageResource(R.drawable.dot_green);
            mEditTextCommand.setEnabled(true);
            mButtonSend.setEnabled(true);
        }
        else {
            mButtonConnect.setText(R.string.stringConnect);
            mImageViewIcon.setImageResource(R.drawable.dot_gray);
            mEditTextCommand.setEnabled(false);
            mButtonSend.setEnabled(false);
        }
    }

    private void showConnectDialog() {

        mDialogConnect = new Dialog(this);
        mDialogConnect.requestWindowFeature(Window.FEATURE_NO_TITLE);
        mDialogConnect.setCancelable(true);
        mDialogConnect.setContentView(R.layout.dialog_connect);
        Window window = mDialogConnect.getWindow();
        if(window != null)
            window.setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));

        final EditText editViewAddress = (EditText)mDialogConnect.findViewById(R.id.editViewAddress);
        final EditText editViewPort = (EditText)mDialogConnect.findViewById(R.id.editViewPort);

        final SharedPreferences settings = getSharedPreferences("TinyTelnet", 0);
        editViewAddress.setText(settings.getString("address", ""));
        editViewPort.setText(settings.getString("port", ""));

        Button buttonConnect = (Button)mDialogConnect.findViewById(R.id.buttonConnect);
        buttonConnect.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                mServerAddress = editViewAddress.getText().toString();
                if(mServerAddress.equals("")) {
                    editViewAddress.requestFocus();
                    return;
                }

                mServerPort = editViewPort.getText().toString();
                if(mServerPort.equals("")) {
                    editViewPort.requestFocus();
                    return;
                }

                SharedPreferences.Editor editor = settings.edit();
                editor.putString("address", mServerAddress);
                editor.putString("port", mServerPort);
                editor.apply();

                new ConnectTask().execute();

                mDialogConnect.dismiss();
            }
        });

        Button buttonCancel = (Button)mDialogConnect.findViewById(R.id.buttonCancel);
        buttonCancel.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v)
            {
                mDialogConnect.dismiss();
            }
        });

        mDialogConnect.show();
    }

    private class ConnectTask extends AsyncTask<Void, Void, Boolean> {

        private BusyDialog mBusyDialog = new BusyDialog(MainActivity.this);

        @Override
        protected void onProgressUpdate(Void... arg0) {
            if (mBusyDialog != null) {
                // Show dialog if it wasn't shown yet or was removed on configuration (rotation) change
                if (!mBusyDialog.isShowing())
                    mBusyDialog.show();
            }
        }

        @Override
        protected void onPreExecute() {
            if (mBusyDialog != null) {
                mBusyDialog.setCancelable(false);
                mBusyDialog.show();
            }
        }

        @Override
        protected void onPostExecute(Boolean result) {
            if (mBusyDialog != null) {
                mBusyDialog.dismiss();
                mBusyDialog = null;
            }

            updateConnectionStatus(result);
        }

        @Override
        protected Boolean doInBackground(Void... arg0) {
            if(!mConnection.isConnected()) {
                try {
                    //mConnection.connect("217.41.71.142", 7373);
                    //mConnection.connect("178.216.9.11", 2049);

                    mConnection.connect(mServerAddress, Integer.parseInt(mServerPort));
                    mInputBuffer = mConnection.getInput();

                    Thread.sleep(500);

                    return true;
                }
                catch (Exception e) {
                    e.printStackTrace();
                }
            }

            return false;
        }
    }
}